@@ -3,16 +3,18 @@ use warnings;
 
 package JMAP::Tester;
 # ABSTRACT: a JMAP client made for testing JMAP servers
-$JMAP::Tester::VERSION = '0.009';
+$JMAP::Tester::VERSION = '0.010';
 use Moo;
 
 use Encode qw(encode_utf8);
 use HTTP::Request;
+use JMAP::Tester::Abort 'abort';
 use JMAP::Tester::Logger::Null;
 use JMAP::Tester::Response;
 use JMAP::Tester::Result::Auth;
 use JMAP::Tester::Result::Download;
 use JMAP::Tester::Result::Failure;
+use JMAP::Tester::Result::Logout;
 use JMAP::Tester::Result::Upload;
 use Module::Runtime ();
 use Safe::Isa;
@@ -120,7 +122,7 @@ has ua => (
 );
 
 sub _set_cookie {
-  my ($self, $name, $value) = @_;
+  my ($self, $name, $value, $arg) = @_;
 
   Carp::confess("can't _set_cookie without api_uri configured")
     unless $self->has_api_uri;
@@ -132,10 +134,10 @@ sub _set_cookie {
     $name,
     $value,
     '/',
-    $uri->host,
+    $arg->{domain} // $uri->host,
     $uri->port,
     0,
-    0,
+    ($uri->port == 443 ? 1 : 0),
     86400,
     0,
   );
@@ -242,6 +244,7 @@ sub request {
     POST => $self->api_uri,
     [
       'Content-Type' => 'application/json',
+      $self->_maybe_auth_header,
     ],
     $json,
   );
@@ -327,6 +330,7 @@ sub upload {
     POST => $self->upload_uri,
     [
       'Content-Type' => $mime_type,
+      $self->_maybe_auth_header,
     ],
     $$blob_ref,
   );
@@ -403,7 +407,10 @@ sub download {
     $uri =~ s/\{$param\}/$value/g;
   }
 
-  my $get = HTTP::Request->new(GET => $uri);
+  my $get = HTTP::Request->new(
+    GET => $uri,
+    $self->_maybe_auth_header,
+  );
 
   $self->ua->set_my_handler(request_send => sub {
     my ($req) = @_;
@@ -413,7 +420,7 @@ sub download {
     return;
   });
 
-  my $res = $self->ua->get($uri);
+  my $res = $self->ua->request($get);
 
   $self->_logger->log_download_response({
     http_response => $res,
@@ -436,6 +443,18 @@ sub download {
 #pod
 #pod =cut
 
+sub _maybe_auth_header {
+  my ($self) = @_;
+  return ($self->_access_token
+          ? (Authorization => "Bearer " . $self->_access_token)
+          : ());
+}
+
+has _access_token => (
+  is  => 'rw',
+  init_arg => undef,
+);
+
 sub simple_auth {
   my ($self, $username, $password) = @_;
 
@@ -452,7 +471,7 @@ sub simple_auth {
   });
 
   my $start_res = $self->ua->post(
-    $self->authentication_uri->as_string,
+    $self->authentication_uri,
     [
       'Content-Type' => 'application/json; charset=utf-8',
       'Accept'       => 'application/json',
@@ -483,7 +502,7 @@ sub simple_auth {
   });
 
   my $next_res = $self->ua-post(
-    $self->authentication_uri->as_string,
+    $self->authentication_uri,
     [
       'Content-Type' => 'application/json; charset=utf-8',
       'Accept'       => 'application/json',
@@ -500,11 +519,10 @@ sub simple_auth {
 
   my $auth_struct = $self->json_decode( $next_res->decoded_content );
 
-  $self->{access_token} = $auth_struct->{accessToken};
+  abort("no accessToken in client session object")
+    unless $auth_struct->{accessToken};
 
-  $self->ua->default_header(
-    'Authorization' => "Bearer $auth_struct->{accessToken}"
-  );
+  $self->_access_token($auth_struct->{accessToken});
 
   my $auth = JMAP::Tester::Result::Auth->new({
     http_response => $next_res,
@@ -538,6 +556,43 @@ sub _update_uris_from_auth {
   return;
 }
 
+#pod =method logout
+#pod
+#pod   $tester->logout;
+#pod
+#pod This method attempts to log out from the server by sending a C<DELETE> request
+#pod to the authentication URI.
+#pod
+#pod =cut
+
+sub logout {
+  my ($self) = @_;
+
+  # This is fatal, not a failure return, because it reflects the user screwing
+  # up, not a possible JMAP-related condition. -- rjbs, 2017-02-10
+  Carp::confess("can't logout: no authentication_uri configured")
+    unless $self->has_authentication_uri;
+
+  my $logout_res = $self->ua->delete(
+    $self->authentication_uri,
+    [
+      'Content-Type' => 'application/json; charset=utf-8',
+      'Accept'       => 'application/json',
+    ],
+  );
+
+  if ($logout_res->code == 204) {
+    return JMAP::Tester::Result::Logout->new({
+      http_response => $logout_res,
+    });
+  }
+
+  return JMAP::Tester::Result::Failure->new({
+    ident => "failed to log out",
+    http_response => $logout_res,
+  });
+}
+
 1;
 
 __END__
@@ -552,7 +607,7 @@ JMAP::Tester - a JMAP client made for testing JMAP servers
 
 =head1 VERSION
 
-version 0.009
+version 0.010
 
 =head1 OVERVIEW
 
@@ -683,6 +738,13 @@ result|JMAP::Tester::Result::Download>.
 
   my $auth_struct = $tester->simple_auth($username, $password);
 
+=head2 logout
+
+  $tester->logout;
+
+This method attempts to log out from the server by sending a C<DELETE> request
+to the authentication URI.
+
 =head1 AUTHOR
 
 Ricardo SIGNES <rjbs@cpan.org>
