@@ -2,7 +2,7 @@ use v5.10.0;
 
 package JMAP::Tester::Response;
 # ABSTRACT: what you get in reply to a succesful JMAP request
-$JMAP::Tester::Response::VERSION = '0.009';
+$JMAP::Tester::Response::VERSION = '0.010';
 use Moo;
 with 'JMAP::Tester::Role::Result';
 
@@ -95,7 +95,10 @@ has para_indices => (is => 'bare', accessor => '_para_indices');
 
 sub sentence {
   my ($self, $n) = @_;
-  return unless my $triple = $self->_struct->[$n];
+
+  abort("there is no sentence for index $n")
+    unless my $triple = $self->_struct->[$n];
+
   return JMAP::Tester::Response::Sentence->new({
     triple => $triple,
     _json_typist => $self->_json_typist,
@@ -146,7 +149,9 @@ sub single_sentence {
 sub paragraph {
   my ($self, $n) = @_;
 
-  return unless my $indices = $self->_para_indices->[$n];
+  abort("there is no paragraph for index $n")
+    unless my $indices = $self->_para_indices->[$n];
+
   my @triples = @{ $self->_struct }[ @$indices ];
   return JMAP::Tester::Response::Paragraph->new({
     sentences => [ map {;
@@ -170,8 +175,10 @@ sub paragraph {
 sub assert_n_paragraphs {
   my ($self, $n) = @_;
 
-  return unless my @para_indices = @{ $self->_para_indices };
-  if (defined $n and @para_indices != $n) {
+  Carp::confess("no paragraph count given") unless defined $n;
+
+  my @para_indices = @{ $self->_para_indices };
+  if ($n and @para_indices != $n) {
     abort("expected $n paragraphs but got " . @para_indices)
   }
 
@@ -206,7 +213,11 @@ sub assert_n_paragraphs {
 sub paragraph_by_client_id {
   my ($self, $cid) = @_;
 
-  return unless my $indices = $self->_cid_indices->{$cid};
+  Carp::confess("no client id given") unless defined $cid;
+
+  abort("there is no paragraph for client_id $cid")
+    unless my $indices = $self->_cid_indices->{$cid};
+
   my @triples = @{ $self->_struct }[ @$indices ];
   return JMAP::Tester::Response::Paragraph->new({
     sentences => [ map {;
@@ -281,7 +292,7 @@ JMAP::Tester::Response - what you get in reply to a succesful JMAP request
 
 =head1 VERSION
 
-version 0.009
+version 0.010
 
 =head1 OVERVIEW
 
