@@ -1,7 +1,7 @@
 use v5.10.0;
 package JMAP::Tester::Response::Paragraph;
 # ABSTRACT: a group of sentences in a JMAP response
-$JMAP::Tester::Response::Paragraph::VERSION = '0.009';
+$JMAP::Tester::Response::Paragraph::VERSION = '0.010';
 use Moo;
 
 use JMAP::Tester::Abort 'abort';
@@ -64,8 +64,9 @@ sub sentences { @{ $_[0]->_sentences } }
 #pod =cut
 
 sub sentence {
-  # die on out-of-range?
-  $_[0]->_sentences->[$_[1]];
+  my ($self, $n) = @_;
+  abort("there is no sentence for index $n")
+    unless $self->_sentences->[$n];
 }
 
 #pod =method single
@@ -141,7 +142,7 @@ JMAP::Tester::Response::Paragraph - a group of sentences in a JMAP response
 
 =head1 VERSION
 
-version 0.009
+version 0.010
 
 =head1 OVERVIEW
 
