@@ -2,9 +2,11 @@ use v5.10.0;
 use warnings;
 package JMAP::Tester::Role::Result;
 # ABSTRACT: the kind of thing that you get back for a request
-$JMAP::Tester::Role::Result::VERSION = '0.009';
+$JMAP::Tester::Role::Result::VERSION = '0.010';
 use Moo::Role;
 
+use JMAP::Tester::Abort ();
+
 #pod =head1 OVERVIEW
 #pod
 #pod This is the role consumed by the class of any object returned by JMAP::Tester's
@@ -19,6 +21,18 @@ has http_response => (
   is => 'ro',
 );
 
+sub assert_successful {
+  my ($self) = @_;
+
+  return $self if $self->is_success;
+
+  my $str = $self->can('has_ident') && $self->has_ident
+          ? $self->ident
+          : "JMAP failure";
+
+  die JMAP::Tester::Abort->new($str);
+}
+
 1;
 
 __END__
@@ -33,7 +47,7 @@ JMAP::Tester::Role::Result - the kind of thing that you get back for a request
 
 =head1 VERSION
 
-version 0.009
+version 0.010
 
 =head1 OVERVIEW
 
